/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

#ifndef PlaneDATA_H_
#define PlaneDATA_H_

#include <string>

#include <xmltio/xmltio.hpp>

#include "RegionData.h"
#include "UnitConverter.h"
#include "Point3D.h"
#include "Rotation3D.h"

namespace airobots {

/**
 * This class describes an arbitrary shaped plane in a 3d space. The plane's shape is described by a
 * polygon in the x-y-layer plus a translation and rotation rule. To obtain the real 3d coordinates
 * of the given points, you will have to add a zero in the third dimension (as the polygon describes
 * the shape in the x-y-layer), rotate the points with the normal rotation angles and translate the
 * resulting points with the given translation values.
 * Attention: The translation coordinates should be given as coordinates of the root coordinate
 * system (before rotation)!
 *
 * @author lziegler
 */
class PlaneData: public RegionData {

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	virtual bool operator==(const PlaneData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PlaneData &type);

	PlaneData();
	PlaneData(const PlaneData &data);
	PlaneData(const Point3D origin, const Rotation3D orientation);
	PlaneData(const Point3D origin, const Rotation3D orientation, std::string generator);

	/**
	 * The origin of the plane's local coordinate system in the root coordinate system. Therefore it
	 * is also the origin of the 2D polygon in 3D space.
	 *
	 * @param origin The origin of the plane in 3D space.
	 */
	void setOrigin(const Point3D origin);

	/**
	 * The origin of the plane's local coordinate system in the root coordinate system. Therefore it
	 * is also the origin of the 2D polygon in 3D space.
	 *
	 * @return The origin of the plane in 3D space.
	 */
	Point3D getOrigin() const;

	/**
	 * The orientation of the plane in 3D space. The is the rotation of the plane's local coordinate
	 * system (x and y axis of the polygon plus z axis as normal) around the given origin.
	 *
	 * @param orientation The orientation of the plane as rotation object.
	 */
	void setOrientation(const Rotation3D orientation);

	/**
	 * The orientation of the plane in 3D space. The is the rotation of the plane's local coordinate
	 * system (x and y axis of the polygon plus z axis as normal) around the given origin.
	 *
	 * @return The orientation of the plane as rotation object.
	 */
	Rotation3D getOrientation() const;

private:
	Point3D origin;
	Rotation3D orientation;
};

}

#endif /* PlaneDATA_H_ */
