/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * PointCloud.cpp
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#include "PointCloud.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

PointCloud::PointCloud() : Type() {
	iLU = MILLIMETER;
}

PointCloud::PointCloud(std::string generator) : Type(generator) {
	iLU = MILLIMETER;
}

PointCloud::PointCloud(const PointCloud &pc) {
	iLU = pc.iLU;
	for (unsigned int i = 0; i < pc.points.size(); ++i) {
		Point3D point = pc.points[i];
		points.push_back(
				Point3D(point.getX(iLU), point.getY(iLU), point.getZ(iLU), iLU));
	}
	sensor_origin = pc.sensor_origin;
	rotation = pc.rotation;
}

PointCloud::PointCloud(std::vector<Point3D> pts, Point3D ori,
		Rotation3D rot, LengthUnit u) :
	points(pts), sensor_origin(ori), rotation(rot), iLU(u) {
}

PointCloud::~PointCloud() {
}

string PointCloud::getBaseTag() {
	return "POINTCLOUD";
}

string PointCloud::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>" + Timestamp::getTemplate()
			+ "<SENSOR_ORIGIN x=\"\" y=\"\" z=\"\" />" + "</"
			+ getBaseTag() + ">";
}

void PointCloud::fillInto(Location &loc) const {
	Type::fillInto(loc);

	loc[XPath("SENSOR_ORIGIN/@x")] = sensor_origin.getX(iLU);
	loc[XPath("SENSOR_ORIGIN/@y")] = sensor_origin.getY(iLU);
	loc[XPath("SENSOR_ORIGIN/@z")] = sensor_origin.getZ(iLU);

	Location item(Rotation3D::getTemplate(), Rotation3D::getBaseTag());
	item = rotation;
	loc.add(item);

	int n = points.size();
	for (int i = 0; i < n; i++) {
		Location item2(Point3D::getTemplate(), Point3D::getBaseTag());
		item2 = points[i];
		loc.add(item2);
	}
}

void PointCloud::fromLocation(const Location &loc, PointCloud &pc) {
	Type::fromLocation(loc, pc);

	for (XPathIterator i = XPath(Rotation3D::getBaseTag()).evaluate(loc); i; ++i) {
		Location pointLocation(*i);
		Rotation3D p = extract<Rotation3D> (pointLocation);
		pc.setRotation(p);
	}

	for (XPathIterator i = XPath(Point3D::getBaseTag()).evaluate(loc); i; ++i) {
		Location pointLocation(*i);
		Point3D p = extract<Point3D> (pointLocation);
		pc.addPoint(p);
	}

	double orix = extract<double> (loc[XPath("SENSOR_ORIGIN/@x")]);
	double oriy = extract<double> (loc[XPath("SENSOR_ORIGIN/@y")]);
	double oriz = extract<double> (loc[XPath("SENSOR_ORIGIN/@z")]);

	Point3D ori = Point3D(orix, oriy, oriz, pc.iLU);
	pc.setSensorOrigin(ori);
}

void PointCloud::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool PointCloud::operator==(const PointCloud &other) const {

	if (!(other.sensor_origin == sensor_origin)
			|| !(other.rotation == rotation))
		return false;
	if (other.points.size() != points.size())
		return false;
	for (unsigned int i = 0; i < points.size(); i++) {
		if (!(other.points[i] == points[i]))
			return false;
	}

	return true;
}

std::vector<Point3D> PointCloud::getPoints() {
	return points;
}

void PointCloud::addPoint(Point3D point) {
	points.push_back(point);
}

void PointCloud::setSensorOrigin(Point3D ori) {
	this->sensor_origin = ori;
}

void PointCloud::setRotation(Rotation3D rot) {
	this->rotation = rot;
}

Point3D PointCloud::getSensorOrigin() {
	return sensor_origin;
}

Rotation3D PointCloud::getRotation() {
	return rotation;
}
