/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/SlamUpdateData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(SlamUpdateDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		SlamUpdateData original("test", true);
		original.setTimestamp(Timestamp(123, 456));
		original.addAddition(SlamUpdateData::metricCoord_t(234.6, 12.0));
		original.addAddition(SlamUpdateData::metricCoord_t(-0.56, 42.456));
		original.addAddition(SlamUpdateData::metricCoord_t(0.0, 0.0));
		original.addRemoval(SlamUpdateData::metricCoord_t(2314.6, 121.0));
		original.addRemoval(SlamUpdateData::metricCoord_t(-10.56, 412.456));
		original.addRemoval(SlamUpdateData::metricCoord_t(0.0, 0.5));
		original.addRemoval(SlamUpdateData::metricCoord_t(1232.0, -45.0));

		Location originalLoc(SlamUpdateData::getTemplate(), SlamUpdateData::getBaseTag());
		originalLoc = original;

		SlamUpdateData parsed = extract<SlamUpdateData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.isInit(), original.isInit());
		BOOST_REQUIRE_EQUAL(parsed.getAdditions().size(), original.getAdditions().size());
		BOOST_REQUIRE_EQUAL(parsed.getRemovals().size(), original.getRemovals().size());

		// compare additions and same order of them
		SlamUpdateData::list_t::const_iterator parsedIt = parsed.getAdditions().begin();
		SlamUpdateData::list_t::const_iterator originalIt = original.getAdditions().begin();
		while (parsedIt != parsed.getAdditions().end()) {
			SlamUpdateData::metricCoord_t parsedCoord = *parsedIt;
			SlamUpdateData::metricCoord_t originalCoord = *originalIt;

			BOOST_REQUIRE_CLOSE(parsedCoord.first, originalCoord.first, TOLERANCE);
			BOOST_REQUIRE_CLOSE(parsedCoord.second, originalCoord.second, TOLERANCE);

			parsedIt++;
			originalIt++;
		}

		// compare removals and same order of them
		parsedIt = parsed.getRemovals().begin();
		originalIt = original.getRemovals().begin();
		while (parsedIt != parsed.getRemovals().end()) {
			SlamUpdateData::metricCoord_t parsedCoord = *parsedIt;
			SlamUpdateData::metricCoord_t originalCoord = *originalIt;

			BOOST_REQUIRE_CLOSE(parsedCoord.first, originalCoord.first, TOLERANCE);
			BOOST_REQUIRE_CLOSE(parsedCoord.second, originalCoord.second, TOLERANCE);

			parsedIt++;
			originalIt++;
		}

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("SlamUpdate.xml"), SlamUpdateData::getBaseTag());
	SlamUpdateData parsed = extract<SlamUpdateData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());

	BOOST_REQUIRE_EQUAL(2, parsed.getAdditions().size());
	BOOST_REQUIRE_EQUAL(3, parsed.getRemovals().size());

	BOOST_REQUIRE_CLOSE((float) 1.0, parsed.getAdditions().begin()->first, 0.0001);
	BOOST_REQUIRE_CLOSE((float) 2.0, parsed.getAdditions().begin()->second, 0.0001);

	BOOST_REQUIRE_CLOSE((float) 5.0, parsed.getRemovals().begin()->first, 0.0001);
	BOOST_REQUIRE_CLOSE((float) 6.0, parsed.getRemovals().begin()->second, 0.0001);

}

BOOST_AUTO_TEST_CASE(testLocationReuse) {

	const std::string attributeName = "testattribute";
	const std::string elementName = "ADDITIONAL";

	SlamUpdateData original("test", true);
	original.setTimestamp(Timestamp(123, 456));
	original.addAddition(SlamUpdateData::metricCoord_t(234.6, 12.0));
	original.addAddition(SlamUpdateData::metricCoord_t(-0.56, 42.456));
	original.addAddition(SlamUpdateData::metricCoord_t(0.0, 0.0));
	original.addRemoval(SlamUpdateData::metricCoord_t(2314.6, 121.0));
	original.addRemoval(SlamUpdateData::metricCoord_t(-10.56, 412.456));
	original.addRemoval(SlamUpdateData::metricCoord_t(0.0, 0.5));
	original.addRemoval(SlamUpdateData::metricCoord_t(1232.0, -45.0));

	Location originalLoc(SlamUpdateData::getTemplate(), SlamUpdateData::getBaseTag());
	originalLoc = original;

	cout << "adding additional attribute and elements" << endl;
	originalLoc[attributeName.c_str()] = 23;
	Location additionalElement("<" + elementName + "/>", elementName);
	originalLoc.add(additionalElement);
	cout << "changed document" << endl;

	cout << originalLoc.getDocumentText(true);

	// do some parsing and re-serialization into the old location
	SlamUpdateData parsedFirstTime = extract<SlamUpdateData>(originalLoc);
	originalLoc = parsedFirstTime;
	SlamUpdateData parsedSecondTime = extract<SlamUpdateData>(originalLoc);

	cout << originalLoc.getDocumentText(true);

	// ensure that the parsed data is still the same and additional elements in
	// the xml still exist
	BOOST_REQUIRE_EQUAL(original.getAdditions().size(), parsedSecondTime.getAdditions().size());
	BOOST_REQUIRE_EQUAL(original.getRemovals().size(), parsedSecondTime.getRemovals().size());
	originalLoc[XPath("@" + attributeName)];
	originalLoc[XPath(elementName)];

}

BOOST_AUTO_TEST_SUITE_END()
