/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.common;

import java.awt.Point;
import java.util.Iterator;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;

/**
 * This is a serializable and iterable wrapper for {@link java.awt.Polygon}.
 * 
 * @author lziegler
 */
public class Polygon extends java.awt.Polygon implements Iterable<Point> {

	/**
	 * Serial version uid
	 */
	private static final long serialVersionUID = 919857141428426489L;

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "POLYGON";
	}

	/**
	 * Getter for the xml item tag used for this (de-)serialization of the items
	 * this polygon consists of.
	 * 
	 * @return xml item tag
	 */
	public static String getItemTag() {
		return "POINT";
	}

	/**
	 * Constructs a {@link Polygon} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Polygon} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static Polygon fromElement(Element objectElement) {

		Polygon poly = new Polygon();

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new IllegalArgumentException("Parsing Error: Base tag "
						+ "for polygon object must be '" + getBaseTag() + "'");
			}

			Elements points = objectElement.getChildElements(getItemTag());
			for (int i = 0; i < points.size(); i++) {
				Element point = points.get(i);
				int x = Integer.parseInt(point.getAttribute("x").getValue());
				int y = Integer.parseInt(point.getAttribute("y").getValue());

				poly.addPoint(x, y);
			}

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(
					"could not parse coordinate values");
		}
		return poly;
	}

	/**
	 * Serializes the {@link Polygon} object into a given XOM {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {
		for (int i = 0; i < npoints; i++) {
			Element item = new Element(getItemTag());
			item.addAttribute(new Attribute("x", String.valueOf(xpoints[i])));
			item.addAttribute(new Attribute("y", String.valueOf(ypoints[i])));
			parent.appendChild(item);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[POLYGON";
		for (int i = 0; i < npoints; i++) {
			info += " (" + xpoints[i] + "," + ypoints[i] + ")";
		}
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof Polygon))
				return false;

			Polygon other = (Polygon) obj;

			if (npoints != other.npoints)
				return false;

			for (int i = 0; i < npoints; i++) {
				if (xpoints[i] != other.xpoints[i]
						|| ypoints[i] != other.ypoints[i]) {
					return false;
				}
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * Adds a {@link Point} to the polygon.
	 * 
	 * @param p
	 *            The {@link Polygon} instance to add.
	 */
	public void addPoint(Point p) {
		addPoint(p.x, p.y);
	}

	/**
	 * Returns the centroid of this polygon. Implemented as arithmetic mean
	 * value of all polygon points.
	 * 
	 * @return The centroid of this polygon.
	 */
	public Point getCentroid() {

		int x = 0;
		int y = 0;
		int count = 0;
		for (Point p : this) {
			x += p.x;
			y += p.y;
			count++;
		}

		if (count == 0) {
			return null;
		} else {
			return new Point((int) Math.round(x / (double) count),
					(int) Math.round(y / (double) count));
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<Point> iterator() {
		return new Iterator<Point>() {

			private int iterator = 0;
			private boolean removed = true;

			/**
			 * {@inheritDoc}
			 */
			@Override
			public boolean hasNext() {
				return iterator < npoints - 1;
			}

			/**
			 * {@inheritDoc}
			 */
			@Override
			public Point next() {
				iterator++;
				removed = false;
				return new Point(xpoints[iterator], ypoints[iterator]);
			}

			/**
			 * {@inheritDoc}
			 */
			@Override
			public void remove() {
				if (!removed) {
					int[] newxpoints = new int[npoints - 1];
					int[] newypoints = new int[npoints - 1];
					int count = 0;
					for (int i = 0; i < npoints; i++) {
						if (i != iterator) {
							newxpoints[count] = xpoints[i];
							newypoints[count] = ypoints[i];
							count++;
						}
					}
					xpoints = newxpoints;
					ypoints = newypoints;
					npoints--;
					iterator--;
					removed = true;
				}
			}
		};
	}
}
