/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.Point3D;
import de.unibi.airobots.btl.common.Rotation3D;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * This is a serializable pointcloud type.
 * 
 * @author plueckin
 */
public class PointCloud extends Type {

	private static LengthUnit iLU = LengthUnit.MILLIMETER;

	public static String SENSORORIGIN_TAG = "SENSOR_ORIGIN";

	private List<Point3D> points = new List<Point3D>(Point3D.class);
	private Point3D sensor_origin;
	private Rotation3D rotation;

	/**
	 * Copy-constructor.
	 * 
	 * @param c
	 *            The instance to create a copy from.
	 */
	public PointCloud(PointCloud pc) {

		for (Point3D point : pc.getPoints()) {
			this.points.add(new Point3D(point.getX(iLU), point.getY(iLU), point
					.getZ(iLU), iLU));
		}
		this.sensor_origin = pc.getSensorOrigin();
		this.rotation = pc.getRotation();
	}

	/**
	 * Constructor.
	 * 
	 * @param position
	 *            Center of the cloud.
	 * @param points
	 *            List of 3D Points building the cloud.
	 * @param sensor_origin
	 *            Origin of the sensor.
	 * @param sensor_rotation
	 *            Rotation of the sensor.
	 * @param height
	 *            Height of the cloud.
	 * @param width
	 *            Width of the cloud.
	 * @param depth
	 *            Approximate depth of the cloud.
	 * @param lengthUnit
	 *            Unit of the height and radius values.
	 */
	public PointCloud(List<Point3D> points,
			Point3D sensor_origin, Rotation3D sensor_rotation, double height,
			double width, double depth, LengthUnit lengthUnit) {
		
		for (Point3D point : points) {
			this.points.add(new Point3D(point.getX(iLU), point.getY(iLU), point
					.getZ(iLU), iLU));
		}
		this.sensor_origin = sensor_origin;
		this.rotation = sensor_rotation;
	}

	/**
	 * Default constructor.
	 */
	public PointCloud() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "POINTCLOUD";
	}

	/**
	 * Constructs a {@link PointCloud} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link PointCloud} object containing all the information
	 *         given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement, PointCloud pc)
			throws ParsingException {

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new ParsingException("Parsing Error: Base tag "
						+ "for pointcloud must be '" + getBaseTag() + "'");
			}

			Type.fromElement(objectElement, pc);

			Elements points = objectElement.getChildElements(Point3D
					.getBaseTag());
			for (int i = 0; i < points.size(); i++) {
				Element point = points.get(i);
				Point3D p = new Point3D();
				Point3D.fromElement(point, p);
				pc.addPoint(p);
			}

			Element srot = objectElement.getFirstChildElement(Rotation3D
					.getBaseTag());
			Rotation3D sensor_rotation = new Rotation3D();
			Rotation3D.fromElement(srot, sensor_rotation);

			Element sori = objectElement.getFirstChildElement(SENSORORIGIN_TAG);
			double spx = Double.parseDouble(sori.getAttribute("x").getValue());
			double spy = Double.parseDouble(sori.getAttribute("y").getValue());
			double spz = Double.parseDouble(sori.getAttribute("z").getValue());

			pc.setSensorOrigin(new Point3D(spx, spy, spz, iLU));
			pc.setRotation(sensor_rotation);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values");
		}
	}

	/**
	 * Serializes the {@link PointCloud} object into a given XOM {@link Element}
	 * .
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Elements oldPoints = parent.getChildElements(Point3D.getBaseTag());
		for (int i = 0; i < oldPoints.size(); i++) {
			parent.removeChild(oldPoints.get(i));
		}

		for (Point3D p : points) {
			Element item = new Element(Point3D.getBaseTag());
			p.fillInto(item);
			parent.appendChild(item);
		}

		Elements oldRot = parent.getChildElements(Rotation3D.getBaseTag());
		for (int i = 0; i < oldRot.size(); i++) {
			parent.removeChild(oldRot.get(i));
		}

		Element rot = new Element(Rotation3D.getBaseTag());
		rotation.fillInto(rot);
		parent.appendChild(rot);

		Elements oldSensorOrig = parent.getChildElements(SENSORORIGIN_TAG);
		for (int i = 0; i < oldSensorOrig.size(); i++) {
			parent.removeChild(oldSensorOrig.get(i));
		}
		Element sensororig = new Element(SENSORORIGIN_TAG);
		sensororig.addAttribute(new Attribute("x", String.valueOf(sensor_origin
				.getX(iLU))));
		sensororig.addAttribute(new Attribute("y", String.valueOf(sensor_origin
				.getY(iLU))));
		sensororig.addAttribute(new Attribute("z", String.valueOf(sensor_origin
				.getZ(iLU))));
		parent.appendChild(sensororig);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[" + getBaseTag();
		info += " points=";
		for (Point3D p : points) {
			info += " " + p.toString();
		}
		;
		info += ";";
		info += " sensor_origin=" + sensor_origin + ";";
		info += " rotation=" + rotation + ";";
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof PointCloud))
				return false;

			PointCloud other = (PointCloud) obj;
			if (points.size() != other.getPoints().size()) {
				return false;
			}

			for (int i = 0; i < points.size(); i++) {
				if (!points.get(i).equals(other.getPoints().get(i))) {
					return false;
				}
			}
			return other.getRotation().equals(getRotation())
					&& other.getSensorOrigin().equals(getSensorOrigin());
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Adds a {@link Point3D} to the pointcloud.
	 * 
	 * @param p
	 *            The {@link Point3D} instance to add.
	 */
	public void addPoint(Point3D point) {
		points.add(point);
	}
	
	public List<Point3D> getPoints() {
		return points;
	}

	public void setPoints(List<Point3D> points) {
		this.points = points;
	}

	public Rotation3D getRotation() {
		return rotation;
	}

	public void setRotation(Rotation3D rotation) {
		this.rotation = rotation;
	}

	public Point3D getSensorOrigin() {
		return sensor_origin;
	}

	public void setSensorOrigin(Point3D origin) {
		this.sensor_origin = origin;
	}
}
