/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * This class is used to store place label info gained from the SwissRanger with
 * svm-based classifier for certain places in a room such as sitting area or
 * desk.
 * 
 * @author fsiepman
 */
public class SRPlaceData extends Type {

	public static final String LABEL_UNKNOWN = "unknown";
	private static final String BASETAG = "SRPLACEDATA";
	private String srPlaceLabel;
	private double srPlaceConfidence;
	private Timestamp srPlaceTimeStamp;

	public SRPlaceData() {
		super.setLegacyParsing(true);

	}

	public SRPlaceData(String lable, double confidence, Timestamp ts) {
		this.srPlaceLabel = lable;
		this.srPlaceConfidence = confidence;
		this.srPlaceTimeStamp = ts;
		super.setLegacyParsing(true);
	}

	public static String getBaseTag() {
		return BASETAG;
	}

	public double getSrPlaceConfidence() {
		return srPlaceConfidence;
	}

	public String getSrPlaceLable() {
		return srPlaceLabel;
	}

	public Timestamp getSrPlaceTimeStamp() {
		return srPlaceTimeStamp;
	}

	public void setSrPlaceConfidence(double srPlaceConfidence) {
		this.srPlaceConfidence = srPlaceConfidence;
	}

	public void setSrPlaceLable(String srPlaceLable) {
		this.srPlaceLabel = srPlaceLable;
	}

	public void setSrPlaceTimeStamp(Timestamp srPlaceTimeStamp) {
		this.srPlaceTimeStamp = srPlaceTimeStamp;
	}

	public boolean isLabelAvailable() {
		return !srPlaceLabel.equals("");
	}

	public boolean isUnknown() {
		return srPlaceLabel.equals(LABEL_UNKNOWN);
	}

	public static void fromElement(Element element, SRPlaceData data) {

		// Checking for TIMESTAMPS
		Type.fromElement(element, data);
		data.setSrPlaceLable(ElementParser.getAttributeValue(element, "label"));
		data.setSrPlaceConfidence(ElementParser.getDoubleAttributeValue(element, "confidence"));
		// TODO: Add FeatureType for SRPlaceLabelData
	}

	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		parent.addAttribute(new Attribute("label", getSrPlaceLable()));
		parent.addAttribute(new Attribute("confidence", String.valueOf(getSrPlaceConfidence())));

		Element tsElement = new Element(Timestamp.getBaseTag());
		parent.appendChild(tsElement);
		getTimestamp().fillInto(parent);
	}

	@Override
	public String toString() {
		return "SRPlaceLabelData label: " + srPlaceLabel
				+ " confidence: " + srPlaceConfidence;
	}
}
