/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * ViewDirection.java
 * 
 * Copyright (C) 2010 Bielefeld University Copyright (C) 2010 Patrick Holthaus
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;

/**
 * Parameterized view angles for the control of a robot head or a pan/tilt
 * camera. <viewangle type="relative" reference="1224498541412"
 * purpose="interact" unit="degrees" pan="-2" tilt="10" inertia="strong" />
 * <viewangle type="absolute" reference="1224498541412" purpose="interact"
 * unit="real" x="1232.2" y="5123.1" z="34.2" inertia="strong" />
 * 
 * @author pholthau
 */
public class ViewDirection extends Type {

	public enum Purpose {
		// scan the environment in a saccadic manner
		scan,
		// focus on something/someone for interaction
		interact
	}

	public enum Inertia {
		// Restrict movement as much as possible, i.e. try to avoid head
		// movement
		strong,
		// Allow other parts like the head to be moved, too
		weak,
		// Every joint can be moved to reach the target
		none
	}

	public enum Type {
		relative, absolute
	}

	public enum Unit {
		degrees, pixels, percentage, real
	}

	public Type type;
	public Unit unit;
	public Inertia inertia;
	public Purpose purpose;
	public float pan, tilt;
	public long reference;

	public static final String UNIT_ATTR_NAME = "unit";
	public static final String TYPE_ATTR_NAME = "type";
	public static final String PURPOSE_ATTR_NAME = "purpose";
	public static final String INERTIA_ATTR_NAME = "inertia";
	public static final String REF_ATTR_NAME = "reference";
	public static final String PAN_ATTR_NAME = "pan";
	public static final String TILT_ATTR_NAME = "tilt";

	/**
	 * Creates a view direction that is parameterized by several attributes.
	 * 
	 * @param type
	 *            Whether the direction is absolute or relative
	 * @param inertia
	 *            Specifies whether as few or as many joints as possible shall
	 *            be used for the movement
	 * @param purpose
	 *            Specifies the intention of the gaze (e.g. scanning around or
	 *            showing attention)
	 * @param reference
	 *            the reference time stamp of the gaze
	 * @param unit
	 *            The view direction's unit (e.g. degrees or pixels)
	 * @param pan
	 *            Horizontal direction
	 * @param tilt
	 *            Vertical direction
	 */
	public ViewDirection(Unit unit, Type type, Inertia inertia,
			Purpose purpose, long reference, float pan, float tilt) {
		this.unit = unit;
		this.type = type;
		this.inertia = inertia;
		this.pan = pan;
		this.tilt = tilt;
		this.purpose = purpose;
		this.reference = reference;
	}

	/**
	 * Creates a view direction in the specified units. All other parameters
	 * fall back to default ones:
	 * <ul>
	 * <li>{@link Type#absolute}</li>
	 * <li>{@link Inertia#none}</li>
	 * <li>{@link Purpose#interact}</li>
	 * <li>{@link System#currentTimeMillis()}</li>
	 * 
	 * @param unit
	 *            The view direction's unit (e.g. degrees or pixels)
	 * @param pan
	 *            Horizontal direction
	 * @param tilt
	 *            Vertical direction
	 * @see ViewDirection#ViewDirection(Unit, Type, Inertia, Purpose, long,
	 *      float, float)
	 */
	public ViewDirection(Unit unit, float pan, float tilt) {
		this(unit, Type.absolute, Inertia.none, Purpose.interact, System
				.currentTimeMillis(), pan, tilt);
	}

	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		final Element viewangle = parent;
		viewangle.addAttribute(new Attribute(UNIT_ATTR_NAME, this.unit.name()));
		viewangle.addAttribute(new Attribute(TYPE_ATTR_NAME, this.type.name()));
		viewangle.addAttribute(new Attribute(PURPOSE_ATTR_NAME, this.purpose
				.name()));
		viewangle.addAttribute(new Attribute(INERTIA_ATTR_NAME, this.inertia
				.name()));
		viewangle.addAttribute(new Attribute(REF_ATTR_NAME, String
				.valueOf(this.reference)));
		viewangle.addAttribute(new Attribute(PAN_ATTR_NAME, String
				.valueOf(this.pan)));
		viewangle.addAttribute(new Attribute(TILT_ATTR_NAME, String
				.valueOf(this.tilt)));

	}

	public static void fromElement(Element element, ViewDirection direction) {
		de.unibi.airobots.btl.data.Type.fromElement(element, direction);
		final ViewDirection.Type type = ViewDirection.Type.valueOf(element
				.getAttributeValue(TYPE_ATTR_NAME));
		final ViewDirection.Purpose purpose = ViewDirection.Purpose
				.valueOf(element.getAttributeValue(PURPOSE_ATTR_NAME));
		final ViewDirection.Unit unit = ViewDirection.Unit.valueOf(element
				.getAttributeValue(UNIT_ATTR_NAME));
		final ViewDirection.Inertia inertia = ViewDirection.Inertia
				.valueOf(element.getAttributeValue(INERTIA_ATTR_NAME));
		final float pan = Float.valueOf(element
				.getAttributeValue(PAN_ATTR_NAME));
		final float tilt = Float.valueOf(element
				.getAttributeValue(TILT_ATTR_NAME));
		final long reference = Long.valueOf(element
				.getAttributeValue(REF_ATTR_NAME));

		direction.type = type;
		direction.purpose = purpose;
		direction.unit = unit;
		direction.inertia = inertia;

		direction.pan = pan;
		direction.tilt = tilt;
		direction.reference = reference;

	}

	public static String getBaseTag() {
		return "viewangle";
	}

	@Override
	public String toString() {
		return "ViewDirection[" + "type=" + type + ", " + "pan=" + pan + ", "
				+ "tilt=" + tilt + ", " + "unit=" + unit + ", " + "inertia="
				+ inertia + ", " + "purpose=" + purpose + ", " + "reference="
				+ reference + "]";
	}

	@Override
	public boolean equals(Object o) {
		if (super.equals(o)) {
			if (o instanceof ViewDirection) {
				ViewDirection vd = (ViewDirection) o;
				return this.unit == vd.unit && this.type == vd.type
						&& this.inertia == vd.inertia && this.pan == vd.pan
						&& this.tilt == vd.tilt;
			}
		}
		return false;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 73 * hash + (this.type != null ? this.type.hashCode() : 0);
		hash = 73 * hash + (this.unit != null ? this.unit.hashCode() : 0);
		hash = 73 * hash + (this.inertia != null ? this.inertia.hashCode() : 0);
		hash = 73 * hash + (this.purpose != null ? this.purpose.hashCode() : 0);
		hash = 73 * hash + Float.floatToIntBits(this.pan);
		hash = 73 * hash + Float.floatToIntBits(this.tilt);
		hash = 73 * hash + (int) (this.reference ^ (this.reference >>> 32));
		return hash;
	}

}
