/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;

import java.util.List;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.asu.ProcessingStatus;
import de.unibi.airobots.btl.data.asu.UtteranceCategory;
import de.unibi.airobots.btl.data.asu.WordNode;

/**
 * Provides test cases for {@link AsuData}.
 * 
 * @author malinke
 */
public class AsuDataTest {

	private static final float RATING = 1.0f;

	private static final UtteranceCategory CATEGORY = UtteranceCategory.description;

	private static final String PLAIN_TEXT = " das ist eine Banane";

	private static final ProcessingStatus PROCESSING_STATUS = ProcessingStatus.fullunderstanding;

	private static final String ASR_BEHAVIOR = "normal";

	@Test
	public void selfCompatibility() throws Exception {

		// default type object
		AsuData original = new AsuData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");

		// content
		original.setAsrBehavior(ASR_BEHAVIOR);
		original.setCategory(CATEGORY);
		original.setPlainText(PLAIN_TEXT);
		original.setProcessingStatus(PROCESSING_STATUS);
		original.setRating(RATING);
		WordNode testNode = new WordNode("hello", "world");
		original.setRootNode(testNode);

		// document
		Document doc = new Document(new Element(Type.getBaseTag()));
		original.fillInto(doc.getRootElement());

		// parsed object
		AsuData parsed = new AsuData();
		AsuData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original, parsed);
		assertEquals(ASR_BEHAVIOR, parsed.getAsrBehavior());
		assertEquals(CATEGORY, parsed.getCategory());
		assertEquals(PLAIN_TEXT, parsed.getPlainText());
		assertEquals(PROCESSING_STATUS, parsed.getProcessingStatus());
		WordNode parsedTestNode = parsed.getRootNode();
		assertEquals(testNode.getWord(), parsedTestNode.getWord());
		assertEquals(testNode.getFrameName(), parsedTestNode.getFrameName());
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("AsuData.xml"));

		AsuData parsed = new AsuData();

		AsuData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), AsuData.getBaseTag());

		// meta info
		assertEquals(PROCESSING_STATUS, parsed.getProcessingStatus());
		assertEquals(ASR_BEHAVIOR, parsed.getAsrBehavior());
		// semantic info
		assertEquals(PLAIN_TEXT, parsed.getPlainText());
		assertEquals(CATEGORY, parsed.getCategory());
		assertEquals(RATING, parsed.getRating(), 0f);
		// semantic tree
		WordNode node = parsed.getRootNode();
		assertEquals("ist", node.getWord());
		assertEquals("Existence", node.getFrameName());
		List<WordNode> nodes = node.getLeaves();
		assertEquals(2, nodes.size());
		assertEquals("Banane", nodes.get(0).getWord());
		assertEquals("das", nodes.get(1).getWord());
		nodes = nodes.get(0).getLeaves();
		assertEquals(1, nodes.size());
		assertEquals("eine", nodes.get(0).getWord());
		assertEquals("Definite", nodes.get(0).getSlotName());

		assertEquals(new Timestamp(1257244616833l), parsed.getTimestamp());
	}

}
