/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.annotation;

import static org.junit.Assert.*;

import java.util.LinkedList;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.annotation.MapArea.MapAreaType;

/**
 * Test cases for {@link Type}.
 * 
 * @author ltwardon
 */
public class MapAnnotationTest {

	private Logger logger = Logger.getLogger(getClass());
	
	@Test
	public void selfCompatibility() {
		
		MapPoint point1 = new MapPoint(0.8, 1.8, MapPoint.Kind.WORLD, MapPoint.Unit.M, "");
		MapPoint point2 = new MapPoint(1.2, 2.2, MapPoint.Kind.WORLD, MapPoint.Unit.M, "");
		MapRegion region = new MapRegion("polygon", "table", "group");
		region.addPoint(point1);
		region.addPoint(point2);
		region.setViewPoint(new NavigationGoalData("test", 1.0, 2.0, 0.0, NavigationGoalData.GoalType.GLOBAL));
		LinkedList<MapRegion> regions = new LinkedList<MapRegion>();
		regions.add(region);
		MapArea area = new MapArea(MapArea.MapAreaType.INTEREST, "arena", regions);
		
		MapAnnotation original = new MapAnnotation();
		original.setGenerator("test");
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.addArea(area);
		
		Document doc = new Document(new Element(MapAnnotation.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		MapAnnotation parsed = new MapAnnotation();
		MapAnnotation.fromElement(doc.getRootElement(), parsed);
		
		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(area.getType(), parsed.getAreasByName("arena").getFirst().getType());
		assertEquals(area.getArenaName(), parsed.getAreasByType(MapArea.MapAreaType.INTEREST).getFirst().getArenaName());
		assertEquals(region.getType(), parsed.getAreas().getFirst().getRegionByLabel("table").getType());
		assertEquals(region.getGroup(), parsed.getAreas().getFirst().getRegionByLabel("table").getGroup());
		assertEquals(region.getHeight(), parsed.getAreas().getFirst().getRegionByLabel("table").getHeight(), 0.0001);
		assertEquals(point1.getKind(), parsed.getAreas().getFirst().getRegionByLabel("table").getPoints().getFirst().getKind());
		assertEquals(point1.getUnit(), parsed.getAreas().getFirst().getRegionByLabel("table").getPoints().getFirst().getUnit());
		assertEquals(point1.getRef(), parsed.getAreas().getFirst().getRegionByLabel("table").getPoints().getFirst().getRef());
		assertEquals(point2.getX(), parsed.getAreas().getFirst().getRegionByLabel("table").getPoints().getLast().getX(), 0.0001);
		assertEquals(point2.getY(), parsed.getAreas().getFirst().getRegionByLabel("table").getPoints().getLast().getY(), 0.0001);
		assertEquals(region.getViewPoint().getX(), parsed.getAreas().getFirst().getRegionByLabel("table").getViewPoint().getX(), 0.0001);
		assertEquals(region.getViewPoint().getY(), parsed.getAreas().getFirst().getRegionByLabel("table").getViewPoint().getY(), 0.0001);
		assertEquals(region.getViewPoint().getYaw(), parsed.getAreas().getFirst().getRegionByLabel("table").getViewPoint().getYaw(), 0.0001);
		
	}
	
	@Test
	public void fileCompatibility() throws Exception {
		
		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Annotation.xml"));

		MapAnnotation parsed = new MapAnnotation();
		MapAnnotation.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), MapAnnotation
				.getBaseTag());
		
		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(MapArea.MapAreaType.NOGO, parsed.getAreasByName("arena").getFirst().getType());
		assertEquals("arena", parsed.getAreasByType(MapArea.MapAreaType.NOGO).getFirst().getArenaName());
		assertEquals(MapRegion.POLYGONE_TYPE, parsed.getAreas().getFirst().getRegionByLabel("table").getType());
		assertEquals("group", parsed.getAreas().getFirst().getRegionByLabel("table").getGroup());
		assertEquals(0.0, parsed.getAreas().getFirst().getRegionByLabel("table").getHeight(), 0.0001);
		assertEquals(MapPoint.Kind.WORLD, parsed.getAreas().getFirst().getRegionByLabel("chair").getPoints().getFirst().getKind());
		assertEquals(MapPoint.Unit.M, parsed.getAreas().getFirst().getRegionByLabel("chair").getPoints().getFirst().getUnit());
		assertEquals("", parsed.getAreas().getFirst().getRegionByLabel("chair").getPoints().getFirst().getRef());
		assertEquals(-3.65, parsed.getAreas().getLast().getRegionByLabel("start").getPoints().getFirst().getX(), 0.0001);
		assertEquals(1.7, parsed.getAreas().getLast().getRegionByLabel("start").getPoints().getLast().getY(), 0.0001);
		assertEquals(-3.3, parsed.getAreas().getLast().getRegionByLabel("target").getViewPoint().getX(), 0.0001);
		assertEquals(-4.5, parsed.getAreas().getLast().getRegionByLabel("target").getViewPoint().getY(), 0.0001);
		assertEquals(1.3, parsed.getAreas().getLast().getRegionByLabel("target").getViewPoint().getYaw(), 0.0001);
		
	}
	
}
